/*!
// @file		MessagePartEnumerator.hpp
//
// @brief		Implementation of the EmaiMessagePartEnumerator class,
//				responsible for message parts parsing
//
// @version		Mailit for C/C++ 1.0 sample file
//
// @author		(C) 2006 Dacons LLP.
//
// @bug			If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

#ifndef __MESSAGEPARTENUMERATOR_HPP__
#define __MESSAGEPARTENUMERATOR_HPP__

#include <EmaiMessagePartEnumerator.hpp>
#include <vector>

/*!
// @class	MessagePartEnumerator
// @brief	MessagePartEnumerator class
*/
class MessagePartEnumerator : public Emai::MessagePartEnumerator
{

protected:
	Emai::UniString						mSubject;

	Emai::ContactList					mContactFrom;
	Emai::ContactList					mContactTo;
	Emai::ContactList					mContactSender;
	Emai::ContactList					mContactReplyTo;
	Emai::ContactList					mContactBCC;
	Emai::ContactList					mContactCC;

	Emai::MessagePart					mHtmlPart;

	typedef std::vector<Emai::MessagePart>
										MessagePartArray;

	MessagePartArray					mAttachments;

	MessagePartArray					mInlines;

	MessagePartArray					mTextParts;


	/*!
	// @brief	Handle the contact fields receiving
	//
	// @param	inContacts - pointer to the array of the EmaiContact elements
	// @param	inContactsCount - size of the array of the EmaiContact elements
	// @param	inOptions - type of the contact field
	*/
	virtual void						OnContacts(
												Emai::ContactList inContactList,
												EmaiOptions inOptions);

	/*!
	// @brief	Handle the message subject field receiving
	//
	// @param	inSubject - pointer to the subject data of the messaage
	*/
	virtual void						OnSubject(
												EmaiConstUniCharPtr inSubject);

	/*!
	// @brief	Handle the message plain text part receiving
	//
	// @param	inPart - reference to the message text part
	// @param	inPartData - not used
	*/
	virtual void						OnPlainTextPart(
												Emai::MessagePart& inPart,
												const EmaiPartData& inPartData);

	/*!
	// @brief	Handle the message unicode text part receiving
	//
	// @param	inPart - reference to the message unicode text part
	// @param	inPartData - not used
	*/
	virtual void						OnUnicodePlainTextPart(
												Emai::MessagePart& inPart,
												const EmaiPartData& inPartData);

	/*!
	// @brief	Handle the message HTML text part receiving
	//
	// @param	inPart - reference to the meesage HTML part
	// @param	inPartData - not used
	*/
	virtual void						OnHtmlTextPart(
												Emai::MessagePart& inPart,
												const EmaiPartData& inPartData);

	/*!
	// @brief	Handle the message data receiving event
	//
	// @param	inPart - reference to the message data
	// @param	inPartData - reference to the message part data
	*/
	virtual void						OnDataPart(
												Emai::MessagePart& inPart,
												const EmaiPartData& inPartData);

	/*!
	// @brief	Handle the composite message part receive beginning
	//
	// @param	inPartData - reference to the message part data
	// @param	inPartData - not used
	*/
	virtual void						OnCompositePartStart(
												Emai::MessagePart& inPart,
												const EmaiPartData& inPartData);

	/*!
	// @brief	Handle the composite message part receive ending
	//
	// @param	inPart - not used
	// @param	inPartData - reference to the message part data
	*/
	virtual void						OnCompositePartEnd(
												Emai::MessagePart& inPart,
												const EmaiPartData& inPartData);

	/*!
	// @brief	Handle the message part receiving event
	//
	// @param	inPartData - not used
	*/
	virtual void						OnPart(
												const EmaiPartData& inPartData);

	/*!
	// @brief	Return contact storage object
	//
	// @param	inOptions - contact type (EmaiFrom, EmaiTo etc)
	//
	// @return	ContactList object
	*/
	Emai::ContactList&					GetContactStorage(
												EmaiOptions inOptions);

	/*!
	// @brief	Return attachment file name for current part data
	//
	// @param	inPartData - part data struct
	//
	// @return	file name pointer, or NULL if filename is not specified in part
	//				header
	*/
	EmaiConstUniCharPtr					GetFileName(
												const EmaiPartData& inPartData);

	/*!
	// @brief	Generate filename for part which does't have filename specified
	//			in header. Generated filename is based on internal counter saved
	//			as user data in the message part.
	//
	// @param	inPart - message part filename to be generated to.
	//
	// @return	
	*/
	void								GenerateFileName(
												Emai::MessagePart& inPart,
												Emai::UniString& outFileName);
	
	/*!
	// @brief	Return string with backslash at the end
	//
	// @param	ioPath - string with or without path delimiter at the end
	//
	// @return	
	*/
	void								AppendPathDelimiter(
												Emai::UniString& ioPath);
	
	/*!
	// @brief	Opens file with "w+b" mode
	//
	// @param	inFilePath - path of the file to be opened
	//
	// @return	FILE pointer or NULL if file cannot be opened
	*/
	FILE*								OpenAttachmentFile(
												const Emai::UniString& inFilePath);
	
	/*!
	// @brief	Called whenever message parts are about to be enumerated.
	//				Initializes counter for attachments without filename
	*/
	virtual void						PreProcessMessageParts();

	/*!
	// @brief	Called whenever message parts are enumerated.
	//				Removes unused user data from mMessage
	*/
	virtual void						PostProcessMessageParts();

public:
										MessagePartEnumerator(
												const Emai::Message& inMessage):
											Emai::MessagePartEnumerator(inMessage),
											mHtmlPart(NULL, false)
										{
                                        }

	/*!
	// @brief	Return the pointer to the message subject
	//
	// @return	pointer to the message subject
	*/
	const Emai::UniString&				GetSubject();

	/*!
	// @brief	Return list of the message contacts
	//
	// @param	inOptions - type of the message contact
	//
	// @return	ContactList object
	*/
	Emai::ContactList&					GetContacts(
												EmaiOptions inOptions);

	/*!
	// @brief	Return the message plain text data and its length
	//
	// @param	outText - on output contains message plain text
	*/
	void								GetText(
												Emai::String& outText);

	/*!
	// @brief	Return the message Unicode text data and its length
	//
	// @param	outText - on output contains message Unicode plain text
	*/
	void								GetUnicodeText(
												Emai::UniString& outText);

	/*!
	// @brief	Return the message HTML data and its length
	//
	// @param	outHtml - on output contains message HTML text
	*/
	void								GetHtml(
												Emai::String& outHtml);

	/*!
	// @brief	Return the number of elements in the attachments list
	//
	// @return	the number of elements in the attachments list
	*/
	EmaiUint32							GetAttachmentsCount();

	/*!
	// @brief	Return the message attachment data and its length
	//
	// @param	inIndex - index of the attachment to be returned
	// @param	outFileName - attachment name
	// @param	outBuffer - buffer to store the data
	// @param	ioLength - the size of the outBuffer; on exit contains the data length
	*/
	void								GetAttachmentAtIndex(
												const EmaiUint32 inIndex,
												Emai::UniString& outFileName,
												EmaiCharPtr outBuffer,
												EmaiUint32& ioLength);

	/*!
	// @brief	Saves attachment to the file system
	//
	// @param	inIsInline - chose inline or ordinary attachment
	// @param	inIndex - index of the attachment to be saved
	// @param	inPath - file system path for saving
	// @param	inNewName - user defined name for attachment (can be NULL)
	//
	// @return	true if successfully saved, otherwise false
	*/
	bool								SaveAttachmentAtIndex(
												const bool inIsInline,
												const EmaiUint32 inIndex,
												EmaiConstUniCharPtr inPath,
												EmaiConstUniCharPtr inNewName);

	/*!
	// @brief	Return the number of elements in the inline attachments list
	//
	// @return	the number of elements in the inline attachments list
	*/
	EmaiUint32							GetInlinesCount();

	/*!
	// @brief	Returns the message inline attachment data and its length
	//
	// @param	inIndex - index of the inline to be returned
	// @param	outName - inline name
	// @param	outContentID - inline Content ID
	// @param	outBuffer - buffer to store the data
	// @param	ioLength - the size of the outBuffer; on exit contains the data length
	*/
	void								GetInlineAtIndex(
												const EmaiUint32 inIndex,
												Emai::UniString& outName,
												Emai::UniString& outContentID,
												EmaiCharPtr outBuffer,
												EmaiUint32& ioLength);

	/*!
	// @brief	Return the number of elements in the text parts list
	//
	// @return	the number of elements in the text parts list
	*/
	EmaiUint32							GetTextPartsCount();

	/*!
	// @brief	Return the message Unicode text data and its length
	//
	// @param	inIndex - index of the text part to be returned
	// @param	outText - string to store text
	*/
	void								GetUnicodeTextAtIndex(
												const EmaiUint32 inIndex, 
												Emai::UniString& outText);

	/*!
	// @brief	Return the message plain text data and its length
	//
	// @param	inIndex - index of the text part to be returned
	// @param	outText - string to store text
	*/
	void								GetTextAtIndex(
												const EmaiUint32 inIndex,
												Emai::String& outText);
	
};

#endif /*__EMAIMESSAGEPARTENUMERATOR_HPP__*/
