/*
// File:		ImapFolder.cpp
//
// Version:		mail.it library 1.0 sample file
//
// Copyright:	(C) 2007 Dacons LLP.
//
// Bugs:		If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

#include PCH_FILE_NAME
#include "ImapFolder.hpp"
#include <iostream>
#include <algorithm>

#include "ImapMessageData.hpp"


ImapFolder::ImapFolder(
		const EmaiListResponse& inListResponse
	):
	mAttributes(inListResponse.attributes),
	mCustomAttributes(inListResponse.customAttributes, true),
	mDelimiter(inListResponse.delimiter),
	mFolderName(inListResponse.mailboxName),
	mFlags(0),
	mReadOnly(true),
	mExists(0),
	mRecent(0),
	mUnseen(0),
	mPermanentFlags(0),
	/*Actually UIDValidity and next UID values should be stored locally between*/
	/*sessions. We will not do this in our sample*/
	mNextUid(0),
	mUidValilidy(0),
	mSelectedForDeletion(NULL, false)
{
}

ImapFolder::~ImapFolder()
{
	FreeMessageList();
}

void
ImapFolder::SetAttributes(
		EmaiUint32 inAttributes,
		const Emai::StringList& inCustomAttributes
	)
{
	mAttributes = inAttributes;
	mCustomAttributes = inCustomAttributes;
}

void
ImapFolder::SetFlags(
		EmaiUint32 inFlags,
		const Emai::StringList& inCustomFlags
	)
{
	mFlags = inFlags;
	mCustomFlags = inCustomFlags;
}

void
ImapFolder::SetExists(
		EmaiUint32 inExists
	)
{
	mExists = inExists;
}

void
ImapFolder::SetRecent(
		EmaiUint32 inRecent
	)
{
	mRecent = inRecent;
}

void
ImapFolder::SetUnseen(
		EmaiUint32 inUnseen
	)
{
	mUnseen = inUnseen;
}

void
ImapFolder::SetPermanentFlags(
		EmaiUint32 inFlags,
		const Emai::StringList& inCustomFlags
	)
{
	mPermanentFlags = inFlags;
	mPermanentCustomFlags = inCustomFlags;
}

void
ImapFolder::SetNextUid(
		EmaiUint32 inUID
	)
{
	/*New messages arrives*/
	if (mNextUid != inUID)
	{
		/*Fetch new messages*/
	}
	mNextUid = inUID;
}

void
ImapFolder::SetUidValidity(
		EmaiUint32 inUIDValidity
	)
{
	/*We need to re-fetch all messages if UIDValidity value was changed*/
	if (mUidValilidy != inUIDValidity)
	{
		FreeMessageList();
		/*Fetch new messages*/
	}
	mUidValilidy = inUIDValidity;
}

void
ImapFolder::FreeMessageList()
{
	MessageList::iterator iter = mMessages.begin();
	while (iter != mMessages.end())
	{
		delete (*iter);
		++iter;
	}
	mMessages.clear();
	ResetSelectedForDeletion();
}

ImapMessageData*
ImapFolder::CreateMessageData() const
{
	return new ImapMessageData;
}

void
ImapFolder::AddMessageData(
		const EmaiFetchResponse& inResponse
	)
{
	assert(inResponse.messageNumber > 0);
	if (inResponse.messageNumber > mMessages.size())
		mMessages.resize(inResponse.messageNumber);
	
	if (mMessages[inResponse.messageNumber - 1] == NULL)
		mMessages[inResponse.messageNumber - 1] = CreateMessageData();

	mMessages[inResponse.messageNumber - 1]->SetData(inResponse);
}

ImapMessageData*
ImapFolder::GetMessageDataAtIndex(
		EmaiUint32 inMessageNumber
	)
{
	assert(inMessageNumber > 0);
	assert(inMessageNumber <= GetExists());
	if ((inMessageNumber > 0) && (inMessageNumber <= mMessages.size()))
		return mMessages[inMessageNumber - 1];

	return NULL;
}

bool
ImapFolder::CompareData(
		const ImapMessageData* inData1,
		const ImapMessageData* inData2
	)
{
	return inData1->GetUid() < inData2->GetUid();
}

ImapMessageData*
ImapFolder::GetMessageDataByUid(
		const EmaiUint32 inUID
	)
{
	assert((inUID <= reinterpret_cast<ImapMessageData*>(*mMessages.end())->GetUid())
		&& (inUID >= reinterpret_cast<ImapMessageData*>(*mMessages.begin())->GetUid()));
	ImapMessageData messageData(inUID);
	MessageList::iterator location = mMessages.begin();
	location = std::lower_bound(mMessages.begin(), mMessages.end(), &messageData, CompareData);
	if (location != mMessages.end())
		return *location;

	return NULL;
}

void
ImapFolder::MessageDeleted(
		EmaiUint32 inMessageNumber
	)
{
	assert(inMessageNumber > 0);
	assert(inMessageNumber <= GetExists());
	assert(GetExists() >= mMessages.size());
	//Check if the message data was downloaded before
	if (inMessageNumber <= mMessages.size())
	{
		delete mMessages[inMessageNumber - 1];
		MessageList::iterator iter = mMessages.begin();
		std::advance(iter, inMessageNumber - 1);
		mMessages.erase(iter);
	}

	//Modify number of messages which are currently on server
	assert(GetExists() > 0);
	SetExists(GetExists() - 1);
	assert(GetExists() >= mMessages.size());
}

void
ImapFolder::MessagesAppended(
		EmaiUint32 inMessageCount,
		const EmaiMessage* inMessages,
		const Emai::ImapSequenceSet& inUidSequenceSet
	)
{
}

void
ImapFolder::ResetSelectedForDeletion()
{
	mSelectedForDeletion = Emai::ImapSequenceSet(NULL, false);
}

bool 
ImapFolder::IsMarkedForDeletion(
		EmaiUint32 inMessageNumber
	)
{
	if (mSelectedForDeletion != NULL)
	{
		if (mSelectedForDeletion.ContainsNumber(GetMessageDataAtIndex(inMessageNumber)->GetUid()))
			return true;
	}
	return false;
}
